package cn.superdesk.app.inventory.controller;

import cn.superdesk.app.base.model.PageResult;
import cn.superdesk.app.base.model.SelectOptGroup;
import cn.superdesk.app.inventory.model.param.InventoryOperationImportParam;
import cn.superdesk.app.inventory.model.param.InventoryOperationPageQueryParam;
import cn.superdesk.app.inventory.model.param.InventoryOperationParam;
import cn.superdesk.app.inventory.model.vo.InventoryOperationDetailInfoVO;
import cn.superdesk.app.inventory.model.vo.InventoryOperationVO;
import cn.superdesk.app.inventory.service.InventoryOperationService;
import cn.superdesk.libs.springweb.model.WrapperResponse;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 库存管理 web端
 * @author xc
 * @since 2023/11/20 15:57
 */
@RestController
@RequiredArgsConstructor
@Slf4j
@RequestMapping("/web/operation")
public class InventoryOperationController {
    private final InventoryOperationService inventoryOperationService;

    /**
     * 获取操作类型
     * @return 操作类型
     */
    @GetMapping("/operationType")
    public WrapperResponse<List<SelectOptGroup>> getOperationType() {
        return new WrapperResponse<>(200, "查询成功", inventoryOperationService.getOperationType());
    }

    /**
     * 操作库存
     * @param param 操作库存参数
     * @return 操作结果
     */
    @PostMapping
    public WrapperResponse<String> operateInventory(@RequestBody @Validated InventoryOperationParam param) {
        log.info("PC端操作商品库存 param: {}", JSON.toJSONString(param));
        inventoryOperationService.operateInventory(param);
        return new WrapperResponse<>(200, "操作成功");
    }

    /**
     * 分页查询商品库存操作记录
     * @param param
     * @return
     */
    @PostMapping("/list")
    public PageResult<InventoryOperationVO> list(@RequestBody @Validated InventoryOperationPageQueryParam param) {
        log.info("PC端查询商品库存操作记录 param: {}", JSON.toJSONString(param));
        return inventoryOperationService.list(param);
    }

    /**
     * 查询商品库存操作记录详情
     * @param docNo 单据号
     * @return 操作记录详情
     */
    @GetMapping("/{docNo}")
    public WrapperResponse<InventoryOperationDetailInfoVO> detail(@PathVariable String docNo) {
        log.info("PC端查询商品库存操作记录详情 单据号: {}", docNo);
        return new WrapperResponse<>(200, "查询成功", inventoryOperationService.detail(docNo));
    }


    /**
     * 导入商品库存单
     * @param file 文件
     * @param param 参数
     * @return 操作结果
     */
    @PostMapping("/import")
    public WrapperResponse<String> importInventory(@RequestParam("file") MultipartFile file, @Validated InventoryOperationImportParam param) {
        log.info("PC端导入商品库存 param: {}", JSON.toJSONString(param));
        inventoryOperationService.importInventory(file, param);
        return new WrapperResponse<>(200, "操作成功");
    }

    /**
     * 导出库存单
     * @param docNo 库存操作单号
     * @return WrapperResponse
     */
    @PostMapping("/export/{docNo}")
    public WrapperResponse<String> export(@PathVariable("docNo") String docNo) {
        log.info("导出库存单 == 单号: {}", docNo);
        return inventoryOperationService.export(docNo);
    }
}
