package cn.superdesk.app.inventory.controller;

import cn.superdesk.app.inventory.model.param.InventoryWarehouseQueryParam;
import cn.superdesk.app.inventory.model.param.InventoryWarehouseSaveParam;
import cn.superdesk.app.inventory.model.vo.InventoryWarehouseVO;
import cn.superdesk.app.inventory.service.InventoryWarehouseService;
import cn.superdesk.libs.springweb.model.WrapperResponse;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author xc
 * @since 2024/2/23 10:33
 */
@RestController
@RequiredArgsConstructor
@Slf4j
@RequestMapping("/web/warehouse")
public class InventoryWarehouseController {
    private final InventoryWarehouseService inventoryWarehouseService;

    /**
     * 查询仓库列表
     * @param param 查询参数
     * @return 仓库列表数据
     */
    @PostMapping("/list")
    public WrapperResponse<List<InventoryWarehouseVO>> list(@RequestBody @Validated InventoryWarehouseQueryParam param) {
        log.info("查询仓库列表, param : {} ", JSON.toJSONString(param));
        return inventoryWarehouseService.list(param);
    }

    /**
     * 保存仓库信息
     * @param param 参数
     * @return 保存结果
     */
    @PostMapping("/saveAndUpdate")
    public WrapperResponse<String> saveAndUpdate(@RequestBody @Validated InventoryWarehouseSaveParam param) {
        log.info("保存或更新仓库信息, param : {} ", JSON.toJSONString(param));
        return inventoryWarehouseService.saveAndUpdate(param);
    }



}
