package cn.superdesk.app.inventory.service;

import cn.hutool.core.collection.CollUtil;
import cn.superdesk.app.base.model.LoginSession;
import cn.superdesk.app.inventory.dao.entity.InventoryWarehouseEntity;
import cn.superdesk.app.inventory.dao.mapper.InventoryWarehouseEntityMapper;
import cn.superdesk.app.inventory.model.param.InventoryWarehouseQueryParam;
import cn.superdesk.app.inventory.model.param.InventoryWarehouseSaveParam;
import cn.superdesk.app.inventory.model.vo.InventoryWarehouseVO;
import cn.superdesk.app.inventory.util.AdminUtils;
import cn.superdesk.libs.springweb.model.WrapperResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author xc
 * @since 2024/2/23 15:13
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class InventoryWarehouseService {
    private final InventoryWarehouseEntityMapper inventoryWarehouseEntityMapper;


    /**
     * 查询仓库列表
     * @param param 查询参数
     * @return 仓库列表数据
     */
    public WrapperResponse<List<InventoryWarehouseVO>> list(InventoryWarehouseQueryParam param) {
        AdminUtils.hasPermission(param.getOrgId());
        List<InventoryWarehouseVO> warehouseList = new ArrayList<>();

        List<InventoryWarehouseEntity> select = inventoryWarehouseEntityMapper.selectByParam(param);
        if (CollUtil.isNotEmpty(select)) {
            select.forEach(entity -> warehouseList.add(InventoryWarehouseVO.builder()
                    .id(entity.getId())
                    .name(entity.getName())
                    .sequence(entity.getSequence())
                    .address(entity.getAddress())
                    .deleted(entity.getDeleted())
                    .build()));
        }

        return new WrapperResponse<>(200, "查询成功", warehouseList);
    }

    /**
     * 保存仓库信息
     * @param param 保存参数
     * @return 保存结果
     */
    @Transactional(rollbackFor = Exception.class)
    public WrapperResponse<String> saveAndUpdate(InventoryWarehouseSaveParam param) {
        LoginSession loginSession = AdminUtils.hasPermission(param.getOrgId());

        List<InventoryWarehouseVO> list = param.getList();
        if (CollUtil.isNotEmpty(list)) {
            list.forEach(warehouse -> {
                InventoryWarehouseEntity entity = InventoryWarehouseEntity.builder()
                        .compId(param.getCompId())
                        .orgId(param.getOrgId())
                        .merchantId(param.getMerchantId())
                        .name(warehouse.getName())
                        .sequence(warehouse.getSequence())
                        .address(warehouse.getAddress())
                        .deleted(warehouse.getDeleted())
                        .build();
                if (warehouse.getId() == null) {
                    entity.setCreatedBy(loginSession.getUserId());
                    entity.setCreatedTime(new Date());
                    inventoryWarehouseEntityMapper.insert(entity);
                } else {
                    entity.setId(warehouse.getId());
                    entity.setUpdatedBy(loginSession.getUserId());
                    entity.setUpdatedAt(new Date());
                    inventoryWarehouseEntityMapper.updateByPrimaryKeySelective(entity);
                }
            });
        }
        return new WrapperResponse<>(200, "保存成功");
    }
}
